"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
exports.__esModule = true;
exports.processFile = exports.createArgsDigest = void 0;
var _safeSharp = _interopRequireDefault(require("./safe-sharp"));
var _fsExtra = _interopRequireDefault(require("fs-extra"));
var _path = _interopRequireDefault(require("path"));
var _debug = _interopRequireDefault(require("debug"));
var _createContentDigest = require("gatsby-core-utils/create-content-digest");
var _duotone = _interopRequireDefault(require("./duotone"));
var _pluginOptions = require("./plugin-options");
var _sharpError = require("./sharp-error");
const log = (0, _debug.default)(`gatsby:gatsby-plugin-sharp`);

// Try to enable the use of SIMD instructions. Seems to provide a smallish
// speedup on resizing heavy loads (~10%). Sharp disables this feature by
// default as there's been problems with segfaulting in the past but we'll be
// adventurous and see what happens with it on.
_safeSharp.default.simd(true);

// Concurrency is handled in gatsby-worker queue instead
_safeSharp.default.concurrency(1);
const processFile = async (file, transforms, options = {}) => {
  let pipeline;
  try {
    const inputBuffer = await _fsExtra.default.readFile(file);
    pipeline = (0, _safeSharp.default)(inputBuffer, {
      failOn: options.failOn
    });

    // Keep Metadata
    if (!options.stripMetadata) {
      pipeline = pipeline.withMetadata();
    }
  } catch (err) {
    throw new _sharpError.SharpError(`Failed to load image ${file} into sharp.`, err);
  }
  return Promise.all(transforms.map(async transform => {
    try {
      const {
        outputPath,
        args
      } = transform;
      log(`Start processing ${outputPath}`);
      await _fsExtra.default.ensureDir(_path.default.dirname(outputPath));
      const transformArgs = (0, _pluginOptions.healOptions)({
        defaultQuality: options.defaultQuality
      }, args);
      let clonedPipeline = transforms.length > 1 ? pipeline.clone() : pipeline;
      if (transformArgs.trim) {
        clonedPipeline = clonedPipeline.trim(transformArgs.trim);
      }
      if (!transformArgs.rotate) {
        clonedPipeline = clonedPipeline.rotate();
      }

      // Sharp only allows ints as height/width. Since both aren't always
      // set, check first before trying to round them.
      let roundedHeight = transformArgs.height;
      if (roundedHeight) {
        roundedHeight = Math.round(roundedHeight);
      }
      let roundedWidth = transformArgs.width;
      if (roundedWidth) {
        roundedWidth = Math.round(roundedWidth);
      }
      clonedPipeline.resize(roundedWidth, roundedHeight, {
        position: transformArgs.cropFocus,
        fit: transformArgs.fit,
        background: transformArgs.background
      }).png({
        compressionLevel: transformArgs.pngCompressionLevel,
        adaptiveFiltering: false,
        quality: transformArgs.pngQuality || transformArgs.quality,
        force: transformArgs.toFormat === `png`
      }).webp({
        quality: transformArgs.webpQuality || transformArgs.quality,
        force: transformArgs.toFormat === `webp`
      }).tiff({
        quality: transformArgs.quality,
        force: transformArgs.toFormat === `tiff`
      }).avif({
        quality: transformArgs.quality,
        force: transformArgs.toFormat === `avif`
      }).jpeg({
        mozjpeg: options.useMozJpeg,
        quality: transformArgs.jpegQuality || transformArgs.quality,
        progressive: transformArgs.jpegProgressive,
        force: transformArgs.toFormat === `jpg`
      });

      // grayscale
      if (transformArgs.grayscale) {
        clonedPipeline = clonedPipeline.grayscale();
      }

      // rotate
      if (transformArgs.rotate && transformArgs.rotate !== 0) {
        clonedPipeline = clonedPipeline.rotate(transformArgs.rotate);
      }

      // duotone
      if (transformArgs.duotone) {
        clonedPipeline = await (0, _duotone.default)(transformArgs.duotone, transformArgs.toFormat, clonedPipeline);
      }
      try {
        const buffer = await clonedPipeline.toBuffer();
        await _fsExtra.default.writeFile(outputPath, buffer);
      } catch (err) {
        throw new Error(`Failed to write ${file} into ${outputPath}. (${err.message})`);
      }
    } catch (err) {
      if (err instanceof _sharpError.SharpError) {
        // rethrow
        throw err;
      }
      throw new _sharpError.SharpError(`Processing ${file} failed`, err);
    }
    return transform;
  }));
};
exports.processFile = processFile;
const createArgsDigest = args => {
  const argsDigest = (0, _createContentDigest.createContentDigest)(args);
  return argsDigest.slice(-5);
};
exports.createArgsDigest = createArgsDigest;